<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{{ title }}</title>
        <style>
      .highlight { background-color:#FFFF00; }
    </style>
    <link rel="stylesheet" type="text/css" href="https://cdnjs.cloudflare.com/ajax/libs/patternfly/4.0.0-rc.1/css/patternfly.min.css">
    <link rel="stylesheet" type="text/css" href="https://cdnjs.cloudflare.com/ajax/libs/patternfly/4.0.0-rc.1/css/patternfly-additions.min.css">
</head>
<body>

    <h1>Aegis REST API</h1><br/>
    <a href='/console'>development console</a><br/>
    <a href='/chat'>chat</a><br/>

    <h2>Aegis features</h2>
    <a href='/api/v1/cve/suggest/impact'>GET /api/v1/cve/suggest/impact/{cve_id}</a><br/>
    <a href='/api/v1/cve/suggest/cwe'>GET /api/v1/cve/suggest/cwe/{cve_id}</a><br/>
    <a href='/api/v1/cve/identify/PII'>GET /api/v1/cve/identify/pii/{cve_id}</a><br/>

    <a href='/api/v1/cve/rewrite/description'>GET /api/v1/cve/rewrite/description/{cve_id}</a><br/>
    <a href='/api/v1/cve/rewrite/statement'>GET /api/v1/cve/rewrite/statement/{cve_id}</a><br/>

    <a href='/api/v1/cve/explain_cvss_diff'>GET /api/v1/cve/explain_cvss_diff/{cve_id}</a><br/>

    <a href='/api/v1/component/intelligence'>GET /api/v1/component/intelligence/{component_name}</a><br/>

    <h2>Prodsec kb</h2>
    <a href='/api/v1/kb/search'>GET /api/v1/kb/search?query=</a><br/>
    <a href='/api/v1/kb/add-fact'>POST /api/v1/kb/add-fact</a><br/>
    <a href='/api/v1/kb/add-document'>POST /api/v1/kb/add-document</a><br/>

</body>
</html>